/*
 * FreeGLUT Shapes Demo
 *
 * Written by Nigel Stewart November 2003
 *
 * This program is test harness for the sphere, cone 
 * and torus shapes in FreeGLUT.
 *
 * Spinning wireframe and smooth shaded shapes are
 * displayed until the ESC or q key is pressed.  The
 * number of geometry stacks and slices can be adjusted
 * using the + and - keys.
 */

#include <GL/glut.h>

#include <stdlib.h>

static int slices = 16; //total number of slices
static int stacks = 16; //total number of stacks

/* GLUT callback Handlers */

static void 
resize(int width, int height) //resize handler function
{
    const float ar = (float) width / (float) height;
    
    glViewport(0, 0, width, height); //set the viewport

//specify which matrix is the current matrix
//GL_PROJECTION: Applies subsequent matrix operations to the projection matrix stack
    glMatrixMode(GL_PROJECTION);

    glLoadIdentity(); //replace the current matrix with the identity matrix

//multiply the current matrix by a perspective matrix
    glFrustum(-ar, ar, -1.0, 1.0, 2.0, 100.0);
    
//GL_MODELVIEW: Applies subsequent matrix operations to the modelview matrix stack
    glMatrixMode(GL_MODELVIEW); 
    glLoadIdentity() ; 
}

static void 
display(void)
{
//glutGet retrieves simple GLUT state represented by integers
//Number of milliseconds since glutInit called or first call to glutGet(GLUT_ELAPSED_TIME)
    const double t = glutGet(GLUT_ELAPSED_TIME) / 1000.0;
    const double a = t*90.0;   
//glClear: clear buffers to preset values
//GL_COLOR_BUFFER_BIT: Indicates the buffers currently enabled for color writing
//GL_DEPTH_BUFFER_BIT: Indicates the depth buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //Sets the current color in RGB
    glColor3d(1,0,0);
//glPushMatrix, glPopMatrix  push and pop the current matrix stack respectively
    glPushMatrix();
//glTranslate  multiply the current matrix by a translation matrix
        glTranslated(-2.4,1.2,-6);
//glRotate  multiply the current matrix by a rotation matrix
        glRotated(60,1,0,0);       
        glRotated(a,0,0,1);
//glutSolidSphere, glutWireSphere  render a solid or wireframe sphere respectively
        glutSolidSphere(1,slices,stacks);
    glPopMatrix();
    glPushMatrix();
        glTranslated(0,1.2,-6);
        glRotated(60,1,0,0);
        glRotated(a,0,0,1);
        glutSolidCone(1,1,slices,stacks);
    glPopMatrix();

    glPushMatrix();
        glTranslated(2.4,1.2,-6);
        glRotated(60,1,0,0);
        glRotated(a,0,0,1);
        glutSolidTorus(0.2,0.8,slices,stacks);
    glPopMatrix();

    glPushMatrix();
        glTranslated(-2.4,-1.2,-6);
        glRotated(60,1,0,0);
        glRotated(a,0,0,1);
        glutWireSphere(1,slices,stacks);
    glPopMatrix();

    glPushMatrix();
        glTranslated(0,-1.2,-6);
        glRotated(60,1,0,0);
        glRotated(a,0,0,1);
        glutWireCone(1,1,slices,stacks);
    glPopMatrix();

    glPushMatrix();
        glTranslated(2.4,-1.2,-6);
        glRotated(60,1,0,0);
        glRotated(a,0,0,1);
//glutSolidTorus and glutWireTorus render a solid or wireframe torus (doughnut) respectively
        glutWireTorus(0.2,0.8,slices,stacks);
    glPopMatrix();
//glutSwapBuffers swaps the buffers of the current window if double buffered
    glutSwapBuffers();
}


//function to check which key is pressed from the keyboard
static void 
key(unsigned char key, int x, int y) 
{
    switch (key) 
    {
        case 27 : 
        case 'q':
            exit(0);
            break;

        case '+':
            slices++;
            stacks++;
            break;

        case '-':
            if (slices>3 && stacks>3)
            {
                slices--;
                stacks--;
            }
            break;
    }
// glutPostRedisplay marks the current window as needing to be redisplayed
    glutPostRedisplay();
}

static void 
idle(void) //executes when there is no function being performed by the user
{
    glutPostRedisplay();
}

const GLfloat light_ambient[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
const GLfloat light_diffuse[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
const GLfloat light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
const GLfloat light_position[] = { 2.0f, 5.0f, 5.0f, 0.0f };

const GLfloat mat_ambient[]    = { 0.7f, 0.7f, 0.7f, 1.0f };
const GLfloat mat_diffuse[]    = { 0.8f, 0.8f, 0.8f, 1.0f };
const GLfloat mat_specular[]   = { 1.0f, 1.0f, 1.0f, 1.0f };
const GLfloat high_shininess[] = { 100.0f };

/* Program entry point; execution starts from here */

int 
main(int argc, char *argv[])
{
// glutInit is used to initialize the GLUT library
    glutInit(&argc, argv);
//glutInitWindowSize sets the initial window size
    glutInitWindowSize(640,480);
//glutInitWindowPosition sets the initial window position
    glutInitWindowPosition(10,10);
//glutInitDisplayMode sets the initial display mode    
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
//glutCreateWindow creates a top-level window
    glutCreateWindow("FreeGLUT Shapes");
//glutReshapeFunc sets the reshape callback for the current window
    glutReshapeFunc(resize);
//glutDisplayFunc sets the display callback for the current window    
    glutDisplayFunc(display);
//glutKeyboardFunc sets the keyboard callback for the current window    
    glutKeyboardFunc(key);
//glutIdleFunc sets the global idle callback    
    glutIdleFunc(idle);
//glClearColor  specify clear values for the color buffers
    glClearColor(1,1,1,1);
//glEnable  enable or disable server-side GL capabilities    
    glEnable(GL_CULL_FACE);
//glCullFace  specify whether front- or back-facing facets can be culled
    glCullFace(GL_BACK);

    glEnable(GL_DEPTH_TEST);
//glDepthFunc  specify the value used for depth buffer comparisons
    glDepthFunc(GL_LESS);

    glEnable(GL_LIGHT0);
    glEnable(GL_NORMALIZE);
    glEnable(GL_COLOR_MATERIAL);
    glEnable(GL_LIGHTING);
//glLight  set light source parameters
    glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);

    glMaterialfv(GL_FRONT, GL_AMBIENT,   mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE,   mat_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR,  mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
//glutMainLoop enters the GLUT event processing loop
    glutMainLoop();
//This macro expands to a system-dependent integral expression that,
// when used as the argument for function exit,
// should signify that the application was successful
    return EXIT_SUCCESS;
}
